"""
This module provides the main Matrix class for hierarchical data clustering and visualization.
"""

# Import main classes and functions
from .matrix import Matrix


# Export list
__all__ = [
    "Matrix",
]
